"""Avert your eyes :) A whole stack of HTML masquerading as Python. Aargh..

Anyway, we define a few strings (long strings) that hold the HTML served up at various points in 
our backend.

Copyright (c) 2010-2011, SKA South Africa
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of SKA South Africa nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

thumb_html = """
<html>
<head>
 <script>
  var thumbnail_ports = new Array();
  var native_w = new Array();    
  var native_h = new Array();
  function resize_canvas(id, width, height) 
  {
   // do nothing :)    
  }
 </script>
</head>
<body topmargin='0'>
<!--thumbnail_body-->
</body>
</html>
"""

thumb_inner = """
 <canvas draggable="true" id='thumbnail_<id>' style="height: 48px; width: 64px; border: 1px solid grey;">
  <script>
   function gen_thumb<id> () {
    var ax_bb = new Array();
    thumbnail_ports[<id>] = <!--thumbnail_port-->;
    var el = document.getElementById('thumbnail_<id>'); 
    var id = <id>;
    el.setAttribute('draggable', 'true');
    el.addEventListener('dragstart', function (e) {
     e.dataTransfer.effectAllowed = 'copy'; // only dropEffect='copy' will be dropable
     e.dataTransfer.setData('Text', <id>); // required otherwise doesn't work
    }, false);
    var c_t_<id> = document.getElementById('thumbnail_<id>').getContext('2d');
    c_t_<id>.scale(document.getElementById('thumbnail_<id>').width/<!--width-->, document.getElementById('thumbnail_<id>').height/<!--height-->);
    var thumbnail_content_<id> = "<!--thumbnail_content-->";
    eval(thumbnail_content_<id>);
    frame_header();
    top.plot_if_possible(<!--thumbnail_port-->);
   }
   gen_thumb<id>();
  </script>
 </canvas>
"""

base_html = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title>HTML 5 Matplotlib Canvas</title>
    <style type="text/css">
    body
    {
     font-family: sans-serif;
    }
    h1
    {
     font-size: 1.2em;
     font-weight: bold;
    }
    </style>
</head>
<body onmouseup="outSize();" onselectstart="return false" onload="connect_manager();">
<!-- Static images used on this page. B64 encoded to avoid seperate path loading... -->
<!-- base64 inline images shock! Page is not served from a standard webserver, hence the malarky... -->
<img style='display: none;' id="button_close" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAAPzSURBVDiNfZV/TNR1GMdf38/3TuBukHCAgMdxwBJisMMfEZPUtaGxcrFqSVY6kmYO+6NWW+VaK9vcWrlqS5p/iDZna1ZTtv5wWsM0CmZNhUPFkx93hyCdXxCuu+Pu+/1y/YF86YB6tuePz/PZ8/o8n/fz2fOR1u4+HGeBmWTBoxUOHi7MZLXdht1uQwJuj4xzdWSCTs9fXOzxoukzC1MxLQwU52XweeNGUkfGMPUOoP/aheYfAyA3fwV2Zx6Pu4oIPuni9aMX6B8ZT8iX5ioUksTuOhcNVYXQ2obW5110ekIlJQXwcj3fdPVz5GwPM/F4IvDVOhcNOSnEjpwGVf1fmGFmM+bGpzgZiHL4zFUAxNw1n68qInL0NLqmoguJlIYtxDPS0CUSPJ5mxbK9Di0eJ9I3yHjjezxbsZLivIxZoEkWHGzcSPR4G5qmoskSqa88g/WJDaS/2wS2NDQBmoCZB6wsf3MnlroarNs2E3L3MT0WILj3Iz5+cT0mWSBqyh1YRu4Q9njRJbBu3YSlpnJWp+wMbG83QUYaWixG+p4GzM48AFK3bsK2fy+qgMnOKyzrvk5NuQPTGqcN3TOILs3KMnm+i5S1ZSQ5cg1o9hs70UMRklc5DfmmPUOMfnkCVcyuY7904ardjFjtsDHtG0EXoAtQIxFuf3aUqH90XvuVKxJgEc8QvfXNhBUFVQZVhmDPDVz2dESBPZPwnTFDJ01ALBLh2q53CF2/taix04PDdD/dTPiugiowfPLmAIWOLExx4uhIxCXJSPr7+i1igXGQ5cVPRZbRzCZUISWExf184R1WMNuzjWcR9A4TVsap/P4Q1lWFi3jJjlxWn2pBXpmdUGFSWREDvgDisk9hmSMPXUBk8h5To6Os++4QqQ8VG5DQoJ+pGwPG2uK0s+6HFmR7tqGhxVXC1eEJxOUhBXNRAZqAcc8g9hfqSS2dhwX7vVx4bg8XG/Zwr/emEbc67RQ0bTMqtG6opts/gehw+4jl5JDitBMMBHB/9TUD37YZsPbtzQQVhdDUFOdfeo2J+1D/jz9x5ZMWVAHLH6lElJXS4fYh56zZ+sGfAwEadmym/+QpNF1juL0DSZK49OGnhBSFGQEzAtRYFO+Zn4nrOpf2H0SPzyBZkqhq/YK3jv+OMhWeHw67tlTwWFKYP/btR5+OLu7uEiYnJ7HuwPu0Ry20nu2Z7fLc5rFzbtqjFmrbTmCrqkzo4FJuq6qktu0E7VELx865jUOkhRO7OC+DAzvWE+m7wd3OTgLX+lA8sx22PVhEVlkJmdXVpJSUsu/4b/89YP9tJllQU+6gwpFORX46xflZAPT7A/T4J+jxTdDh9i35BfwDr1enT5ZTWAsAAAAASUVORK5CYII=" />
<img style='display: none;' id="button_close_over" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAAPpSURBVDiNfZVdbJNVGMd/57wt68e6jXXrurXrysCVbc6ADBDRhJiISIyLGiEQQ4hTQiY3akz0bnoraDSC4QbjhYmJH8TECyUaFogYPmLG5GNsbF3XfeDat1/vum5r375eDF5WRniS5+L8c57/85z/ec5zxKZDJw0eMKOok4leQ6gR9DujJBOTALhr/Fgb1mG4gzjq1yOk8mAolgeBXGKKZN8ptla6ea7Gx4bHt9NaVQvAzXSMG1qcv6f6uT5xkWJHF8LhLokXZoWGQfrfMxRuXeD45p1s8wRWZF9uA7MqR9UhNE87s+42QAAg721ID/xOe3yE87sPssUbQJc80tsr3JwMbmVHMYl3IWImkveOmR+6wBdPvYjVYkGXAvvenRjVFeiCEjcqnDj27aJgGOjhKfZfuUNTbgq7mF8iNIo6ib5TfLbtBSxWCwVF4HrrVZy7n2X1R93grqAgoSChWOmk6v0DOHZtx7nnebLXbqEn07x2Jcpm6xQCUBw1Tb0duRiH2rdgCHC9tAPXzqeXsjnt2Da2kv3nBoW0Rs17Byl7bEnbspYghtNG+uxFHHOLaI01/LfajaUYD/OMtxFd3NWy7yL2TW2UBeqX2sBTjefdA+jZHLaWoKnV/PAY0199R/7uLQQm4/jXgNRnwrTV1pti53M5Jj//hoXotBls9dWVkOWGx7je1cOcqpJXIK+AU83Q7JLIVGKK5ppaU6eChMVcjhtvfkj25u0V7TIfnmDglR7m4ip5iemWeApvbTkWMNCFQAphBs3evM1iLAHKypeAolCwWshLUYpLid3uQFZW+xhOxc220CITzKkJNvx4HGfLmhV8tkA9G0+fQPF5SirMN9aizYO0NbQwpCXQJeTSKTLT03T+cBxX61qTJBuOkhkcNdeOoJ/On06g+D2mhqzzE51XkJa6Zq4mYhQkJIbD+Pd34Vp/n0wbiXDu9cOc33uY1PUhE3cG/TR17zEr5MkOxtMFpMMbIqzrDCZjaLEY177+ltHvfzHJzu7rQVNVspkMfW8cIXmXNPrrH/R/eoK8BKWtGdrbGIgkURo6X+6lKsD4dD8hbQFDGEyc/QshBJc/PkpWVSlKKErILy4Q+e1PDF3n8ifH0I0ihs1Kfe8HfHluklR28f60caUHaU2O0BmZQRZXjMiHmlxlpeFIN5esXn6+NA4sm4daZYgxhwPhixEaDGNPZB5JVh5aS+M7b3MuOs/py1ETXzZgBTPWAFnpxb69DvdshoqZGDKRwlBTCGCV14OzqZHyDU9QFmrlWF+EaGy2JJF42BcgBfjKizSUw5oqC75aF3abnXTOIJoVjKfzXA0n0B8izf+Xmac2etmdhAAAAABJRU5ErkJggg==" />
<img style='display: none;' id="button_max" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAAN+SURBVDiNfZVbTJtlHMZ/3wGwVBhIWbGxjDMbwnAC0bCDuiWiS3YhlybsZsmmxuiNd8aIBO/MrrxZ4qJREy92pU6NyYynThonDGLHqSml5UxpCy2Utt/3fp8XbKWlxSf5J1/+h+f/vG/ePJ/UdfWGyQGoisyFZxvoaLXT2lhNXdURAIKRGDO+EKMPlvl9Yh5dGAdHkQ4SHnfaGHrrAmuPpXkgx5gXOyyIBABOpZQ6xUqbUc7R3SLe//QOvuVIYUJZknijv4e+s018rvuY0WN527PRqpZzWWngu1+m+fKHcQxzT5f6qOHqa92c6H2SD3fG0BDID/PdxdXUqBYAwiLFSGoNAK+IMMQWAy82cRn44vb9fcIWp41XzjYzFP8bHYGUpeRMiZ1TJdUATKejuBNLAEREEr8W43Zslt9e6OfPsQC+5QiqqsgMv3merxMedD2ZUZaBuX/xpmkwlwjh1zbZFKlM/t3oHa5fe4krQ9+inuusI6Tu4N0K5SjLYsl8RvQEE/EFgJzF7i0/k/Z2TrfXona22pnR1pGEXvDytey8YSBr6YJ9ruQcHY0OpNHAkvmjNEkwFeWU1cG5I00PhZnERYpmiw1bkRWATX2Xse2lHKJbG+Pc2pjgpNXBkOUiav3RSlYDYWTTwKZYeLrUXlABQIVq4XxFU05uNDaPrKWZ3VqgpaEK1TRMEBqSaYAhDiU7DJLQkbUUkqyACerceoSnisoI7oaJJGO4o16SQssMtJc7sRWX7R1ZSzC+NZ9DGNheRdbStJU7mV3ZQJ3ybVDbXMni9hrfL44w7PkmZ+Bm99u8XPMMANOxBQbc1/NUKkCn1cG0L4w8MbNOfbEdSdfxRQPIWjonst8hpplXfxS9jx9nai6M7PIEqaKSmuIykrvxvEbJ2CeUTKMg2XMVDbSUHOOuJ4isC4MPbvzB684+rIaJoqVyQjKzCfPrVsPk445rfPSZC10YKI6uS4PR+C6lxUX0n+zCtehG6GkkQyAZAk1oTEXmGFkZx70ygTfsy9QsksJwzzuMjsb59X5gb2m2fQ30tfPqmWMMuj/hn9V///e5dNd0MPj8e/zkCvDVz56MfeUZbKPjCQav9OKLz3Jv7R6ToVm8YT8AzVX1tFW30GPvobGshcGbfx1usNlQFZnT7bW01Vdyor6SZseefXmXQ0z5o0z6o9z1BAv+Av4D2HLBnnU1sUYAAAAASUVORK5CYII=" />
<img style='display: none;' id="button_max_over" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAANnSURBVDiNhZXLbxtVFMZ/cz22M0mTOK7jR4lD4qRNrZaQqEFRJQQSiLJoQWIFLNghFgiBRP8FlohF6aYLBBJCLAKIBQukSixI2gaJhlSlJTQPbNcxiTNJPI4T2/O6LNranjgSR7rS6Nzzffc7Z865Vzn33jXJYZMOUl/moJSjtJ3BWM8A0D80Sjh2EqElUMIpUEQbVD3sENUt1m58RWhihMgbkwxNXCKUTgFQeZBlfylLfWEVe+UeMv4ibjDkwStPFCpSUs/Nkl+5yXNXLhM9P952eqvV7meoXPsZUx3E8A+3E9Yzv1LrKjJ95WN8HcFGwFSgn7iqAbDt1LlV32yyWjbmN7Ps/eNnw441UxbVLdZXbnDxl8/x+X3g2A3M88EYk8F+AJbMXeYP1gHYdepkrDL51+K8/nWOyq5Fxfaj4jqszX3J+U/fx68KsC1vbtL1fGerOhnTQHeqDffNC8e5OLfPXC6E6mw9IHx2kPjUaXCc9mLJZhPs2FUWyjkAWv/vRghKyQBJQ0GtGTlir6ZRWtJsNbvFL10HYZlHxuVjPk4VVFRDXyOdnkY4NpNdJ3ihd/SxMMmBazIc7GuAzmhRvh9520M0oy8yo9+h2CF4OaGhGoUskVQCYdtEfBpnOmNHKgAIqRovhUY9vtvlDMIyKSoWT0e6UaUExbFQhAT3iBr+jymOjbDqCASapqGGB4YwVvKETw2wUyvze2mNil1rAM72JIkEugEoWQcsGhkPYbaygbBMBkUf25aLGkmMUVkpEBmJc71wm0/+/NYD+GLqAy7EJwBYKj/knfnP2lT6gLGuMBsbDoJjA5TuFVBsm6yRR1imZ3n7ULbtP1nngikeFqsI9fgIZqGK/lcWfa/YFqi4TUJFukeSPROIMxYY4o9VHRVFQPIVdn+6hXoJpO9Q0WUrocRn1T37AeHn8uCbXJ35G9txH82yGwxhypOkFnJkxyu4ojkdPy5f587mfQAK+1uexg4IPx+l3uK3uwbZzfKjQ5sXrCRKgcSxHZaG8mx3VNqK32rp3hQfnn6X+bt7/DC7inw8oi0XrEKRpzio9jO9HaXWW0HXttCVErpbAuBER5Th7iTjfc8y2p3m6nfLDWUNlqOeAKFAsgcSfYJk1M9gpIdOrZOdmsu/JcgV6yyu6diOexjKf/dLl8SuHWsnAAAAAElFTkSuQmCC" />
<img style='display: none;' id="button_resize" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAAOhSURBVDiNfZVLTFxVGMd/99zLMMy0PGZ4F6bA8Ia+tY0lscZGqZFFk+60gQWG7mriWppqWxcmmlTdNLE1ttW40Jq4MEpqQmOpxjYoQkEYec2Y4TUMDJnXfc24GB0GmPaf/Hf3/M7/fN+535GO9F1Lsk2KLOhod9FaX067u4L6ikIAZpZCTMwsMjLpZ2jMi2Emti9F2g50Vzp459zLLCctTEVhLmLii5oAVNtkauwyjTYokzQuXBtg2h/MDhSSRPcrh+k60cbteZ3JDWPH7plqylc4uzeH7wbHuP3D7ySSqVzK/x90nzrE/meaufxnGN0E8VQceIIml0Mqrx9poUeCz74fBv5b5650pJJNhnm+1MJpl5VjTgWXFSwJE8nI7oV1lbfuBzj5XAvuSkcqoSIL3u17ia8mIxhxA2eO4FX3rnSSZBICMZOFiM5C2OCeL8KgN8psSCOspZry5mCAD3tP0vveNygd7S5WVAnPShQJWNhQgU2gJEGJTabEJhNWI9x4uEzMSG4eD/h1NsR4i52OdhdKm7uM6aCGZKQ6ubgez1qzX7xhznzhQTUSCKDYphCIbjbu/nyEfbWliLbacnzBGMIwEYbJ0nqc5I6bCfvL8yjNSSJrOp01dj49XYOs6WmP+jZo3luGaKgoZHE1ijAMhGGgx3XWYvoOoN0i8+D8AXoOOrn5WhP7ym0ITU97yheicU8RApKgm0gZnl2OAvCzZ42YZm6BfnCmAYss4bDn4FRAqDpC1ZFUA0gi/vavU7U7J30VAsEowzPrDI4H6P7kES9cGtoCzVSTIzedsLU4l6l/gojxuSVcRbkIwyQaVnk0scy3Q17e+Pg3jIiKf2GDC18+zgpsLLama3ig0s5f88uI0ekl6kryEKbB9PwaiZjGY08APaIiNJ0Xmx1cOdueFdhQmpdOeLyhkIm5FcTQmBfHboXaYht+f2hLoZ25gr5TbqLx7P+1u3wXQtM5VldI4x47Q2NeFMNM0H/9J94/18n1r0eJZ9RrQ9Xp6b8LgLPASn11AfVV+dRXF+CuKiDfqmBPJrjSe4i3b9zFMBOb06an8yCuIif9Hz3YAn2arBaZS+eP411b5fMf/wAyhsqtgRG8wVXuXO3iaJNzy9Gz+WiTkztXu/AGV7k1MJLeJOuAvdh9gumZNR4O+xmfWsEzu5pqQq2T1sYSnj1cibuuiIs37z15wGYq/QS4immpLqGhOjWaPL4gE74Vxr2BJz4B/wKrEccDtJ7HywAAAABJRU5ErkJggg==" />
<img style='display: none;' id="button_resize_over" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAAOaSURBVDiNfZRLbBtVGIW/uTO2JxkTv52keTZOSBy1oRIVqEARgoqHuinqAlGhdIO6RIJFxQY2iAWLIlWwAAmxABGxqITEgk15BKEQqZQCah4iIS11EqdxjV8Zj+15eFhU2HFs8Utnc6X/3POfe/8jPXzhY5cD5dYdiqklKvktjOI2lUwKAH/vKFp4ADU4iH9gGknIB1tRDh5UcmlSi3OEp2YYeuYUocljBMeSABRurVLaWKKwdI3ijTl6ki8gtGhLv9RQ6LrsLn1HYfM3HnvzMvGZE22376+99T/YmruECI5Rj0wDUivh7s2ryGqdJy5eRvap/0vWsMYy2f36I8ySxV7XYQDEf2MWU79z8o33eXa0hzPDKo9GFIZV8NYdJLsz/qm6bB6fRZOK+Fzjvodu3SG18AVPvv4eHqEQ8QhOJ/xNFS5kKw47ZYsd3ebHzTLzKYPbRRPdrANgTLzI4+mrrDkJlGJqiWhimr6p4+A47JRqQJNQkiDWLRPrltFrZT79JUPFdpvjAXlPL3pgkJheRVRymwwePdEY426h2tGvxZTO2c/XqBkmwrSIKy7CtBq45xsg1O0iqrktYqNJhO0gbIfdQhW37WfCTF8XcY+LbFo8N6rxyZlRZNNqoOD2EH/AgyhnU0T7xxC2jbBtrKpFvmK1EWpemZ9fe4jzxyJ8dm6So33dLQrzNR/9AS9CckGynBbcztx/sZ/W81RMp4X00tkJvLJEWPMQUUDUrAZUnw+hxYbIp9YaHmZzBjduFZhfyTL74XWeemehhXR/TYZ9DYX9coU9x4vQoiMUN/9E2A6GXuP6aoavFlK8+sE17HKN9E6Jt+eWOxI+GFUbHo5oDjkDRFd0mOzfNxGOzcadPPWKyfJ6FqtcQ5gWT0+FefeVIx0JJ+JdDYXJGGR0B6EdSqLn0uTurJJOF1uMjvgEF55PYFTtjoSJPj/CtBgPOIzFvazdNVAkIaMdOc1fi18iQi8hSc0AKtUszr/1LQCRgMr4UIDxwR7GhwIkBgP0qAqqbTN7MsyVXzM4dbcZDkp2GdmokhWP4NKec53Ko0i8fCpEybX5fjkLNLcHOzqNJxwk6V8kKOVaRu+ERFTi4rk4Jdfhh5Vs45J9ASuh+xPYbj/J0DaWpVPUNUqGl4IuIwHRkOBQr5eJw36GR/xcuXGPnQOr2pbYVambFXecmFYlGnGZ1qAvqKCqPkqWh1xNYkO3+GZ+G6fevqP/AuWCyZZYcPwAAAAAAElFTkSuQmCC" />
<img style='display: none;' id="button_home" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAANvSURBVDiNjZVdTJtVGMd/79tauvLZFoaAfJQP040CUmYmA0TishCTGXXJIjEmxk2vjd5suzC7WmI0eqdRkwW3C40X6p2JJBKJjBgWt0ExE6G23cqg3Whrx1ffc87rRUs3GApP8r865/zO85Xn0bre/txkm1ktOj2+Op5uLsfvcfNElRsNiMYSTIcTTPy5xNhUGCHV9qdo24FN1S4+eauPkrUlrLEgcnkBmVwCwFJWicVVjahs5r6jkne+GGMuem9noK5pnB5sZ6i3ESZ/QCyFHrqlgZn9V7PaQNexuGvQDr3I17/O8+WPU6jcuaW66/h5gNOD7bzaaifz8yXkP8uYZpZhahaKn38NbPsw4lGKBoYo7D2BWk2zNvYNHZ1tOFwVTM4uAqBvhjnU28j6+HdII4NUKisTivpPYqv1UtR9HFtzJ2bOE6UU63dCLF88y0v+KpqqXVmg1aLz0aleMhPfIzIGQpKTRnHfCewe32bclPS9gmYvBGB9MUx69gZrsQXSl9/ngzeOYLXoWHt8dThSUVaiwS3JdT37Mo4n/dtKqGPbX5v1UEgMka1y6o+rlEcD9Pjq0DvqnajFIFKSV+nhFyhqfeaRltiEAljd1RjCzCszc4WOeie63+NiPbaQz1tJ1wCl/v6dYQ9Zqf857C2dGFJhSEV6foaO+jL0hpoKVuN3EBKKfUdwdw/uCgPQC+x4z3yGvaEVQ5ik/p7FU1ORrbKUJtI0WY0vEPr2U+Tayq5AUwqunz/Fxv0VDKEQuXzqwdtxHnM9jhQm6XCQ+O8TyI313YFKcTdwjWRoDkOYFNS0ELwdR78WWsZWUYVUio3VFWLTk/le280282dIhaPxADciSfSpcAJLZQNCQTI0T2ZD7AkGbKlyYXsvU+EE+nggguGuw1FVTyoawRCKvfkHhlAYQlF2sAvd0854IIIupOLMV1eoPHYSoXQMYbJXoiFMlKWAg+99yNlLEwipHkybN4/5GChOcPXCu+hWK1qugf/bTKRhcOjcx4ymnVz8KQDkhgPA8MgMo2knR4dHKGk8wEoy9b8q9ng5OjzCaNrJ8MhM/psdB+yF17tZ++s6d3/7hfjNAPfmbgLgbvZS4fVRfriffS1Pce7yBPMLy1v8fgQID1ZAW52TttpSmmr3AzB/K8b0rRTTkQTjgciOK+Bfa+a4GLtdYmQAAAAASUVORK5CYII=" />
<img style='display: none;' id="button_home_over" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAAEAwAABAMBtJq7BgAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoAAANtSURBVDiNjZXdT9tVGMc/59fS/aAvvJettALDAnMYFTTbVGTTGHWJibpsiTOaqIm36oWJd+4/WPRiaqL/gGYX3umlGDSZOnWQiGy8tJXCaPtrKRRafufFi0IHjAhP8tycPOdzvs9znvMcMfzel4Y9ZrSimJyEwjw6N4uTSSOAcKSLhsgAMhjDF46DsPZuxbt3YcNJ44x/zemuFp4djPFo91OciLYD8M+iw99phxupW0xN3aTc8xLabtq1X9QUGkNh8gfk3DjX3n6BM32xHVECTDVMeH1gWUymlvl0fI6V0ACOPw4IADyR4ZevADgT3zNoZrj+wUW62psxpsowwkPwuTfAV4+bWSBw7nX8T1+gPRTgrL9IvpijpARFQvdS3nDSqLmf+eyTt6jzWiitt5RZNI5ewhcbwBfrR7ubmC2lWmt0Ls3FugWWK4Kcp411fQTLaEX2p6+4+ubzeC0vUrHlguDIBeyewe28CY28hrD9AJSXEqxO/4UqZHnF3OZUcBFLgHclOclwJMTjxzvvKQNannmVhr6h3TcmLHzham21VLiyGt8q8/RSYNovsdzMDCP9UZSi5o2nzhM4efq+ltiGAnhbI7jS1DxaXiIaAEvnZnkodhSlNUprQsPnaBwa3R+2wxqHzmLHH8NVGldp/CWHniaBVXQWOR5uQyoIDj5J65kXD4QBWEdsBj7+HLv7JK40eIo5jrUGsIyhqs4Y1jNp5r+5htooHQg0SvLnlXeprJVwpUZKjW3bWC3hGHeWCihpWE3Mkrn5C6pSPhioNdnJPyjM38GVhs1QB6sVsHxH+7i97KC0prJeYnni11qvHWTb9XOVho4HSK15sHzhXm6lHaSGwvwMmxV5KBiw65bFg8MkVhSWHY4zV/YwfTfPykISV2oOpw9cqXGlxhONY7oeZiJRwEJYqPh5vp1eZr0scaXhsERXGqSoo+nyh3wxtoDSpvqWjd1MsfkEN+oMQ9kprl8aRewz63abQVpejl1+n7Gk5N9stTNq87AY7GOuvgETjtI//zv1hez/4vy9A/S88xFjSZfvfluore8YsIK73hhrVgf1Qx20ljIEMyk8+Qwmn0EAvo5OGmI9NAw+gS/+CFd/TJPKru06SOz3BVgCIgFNp9/Q3SiItAWx622KG4ZUyUNyRTGRyKP0/cX+DwdjpAeUBIzPAAAAAElFTkSuQmCC" />
"""

base_html_decoration = """
<span id='title'>
<h1>HTML 5 Canvas Matplotlib Backend</h1>
</span>
<table width="100%" border=0 cellspacing=2 cellpadding=2 style="margin-bottom: 0; position: absolute; left: 8px; top: 38px;">
<tr>
<td style="height: 50px; -moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey;">
<iframe name='thumbnails' id='thumbnails' style='border: none; height: 54px; padding-top: 3px;' width="100%">
 <!--thumbnails-->
</iframe>
</td>

<td width=200>
 <table cellspacing=5 border=0>
 <tr>
  <td>
  <div onclick="set_layout(1);" style="cursor: pointer; -moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 22px; width: 40px;"></div>
  </td>
  <td>
    <table onclick="set_layout(2);" cellpadding=1 cellspacing=0 border=0 style='cursor: pointer;'>
     <tr>
     <td>
      <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 20px; width: 20px;"></div>
     </td>
     <td>
      <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 20px; width: 20px;"></div>
     </td>
    </tr>
    </table>
  </td>
 <td>
   <table onclick="set_layout(4);" cellpadding=1 cellspacing=0 border=0 style='cursor: pointer;'>
     <tr>
     <td>
        <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 10px; width: 20px;"></div>
     <td>
        <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 10px; width: 20px;"></div>
     </tr>
     <tr>
     <td>
        <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 10px; width: 20px;"></div>
     <td>
        <div style="-moz-border-radius: 5px; border-radius: 5px; border: 1px solid grey; height: 10px; width: 20px;"></div> 
    </tr>
    </table>
 </td>
 </tr>
 </table>
</td>
</tr>
</table>
"""

base_html_canvii = """
<span id='plot_canvii'>
<span id='plot_canvas_0' style='position: <!--canvas_position-->; top: <!--canvas_top-->px; left: <!--canvas_left-->px;'>
<canvas ondragover="onDragOver(event);" ondrop="onDrop(event, 0);" style="border: 1px solid grey;" id="canvas_0" width="640" height="480" onmouseup="releaseCanvas(event,0);">
</canvas>
<div id='button_menu_0' style="-moz-border-radius-bottomleft: 5px; border-bottom-left-radius: 5px; -moz-border-radius-bottomright: 5px; border-bottom-right-radius: 5px; background: #336699; border: solid 1px grey;  top: -1px; position: relative; width: 640px; height: 20px;">
 <table width="100%" border=0 cellspacing=0 cellpadding=0>
  <tr width="100%">
   <td width="24" style='padding-left: 2px; padding-right: 10px;'><img onclick='close_plot(0);' onmouseover='this.src=button_close_over.src' onmouseout='this.src=button_close.src' style='cursor: pointer;' id='cb' src=''/><script>document.getElementById('cb').src=document.getElementById('button_close').src</script>
   <td width="65%" onmousedown="clickMove(event,0);" style="cursor: move;"><span id="status_0" style='color: white;'>Disconnected</span>
   <td width="35%" onmousedown="clickMove(event,0);" style="cursor: move;"><span onclick='change_cursor_info(0);' id='cursor_info_0' style='color: white;'>Cursor</span>
   <td width="24" style="text-align: right; padding-left: 5px;"><img onclick='go_home(0);' ondragstart="return false;" onmouseover='this.src=button_home_over.src' onmouseout='this.src=button_home.src' onclick="maximise(0);" style="cursor: pointer;" id="hb" src="" /><script>document.getElementById('hb').src = document.getElementById('button_home').src</script>
   <td width="24" style="text-align: right; padding-left: 5px; padding-right: 5px;"><img ondragstart="return false;" onmouseover='this.src=button_max_over.src' onmouseout='this.src=button_max.src' onclick="maximise(0);" style="cursor: pointer;" id="mb" src="" /> <script>document.getElementById('mb').src = document.getElementById('button_max').src</script>
   <td width="24" style="text-align: right; padding-right: 5px;" onmousedown="clickSize(event,0);"><img border=0 onmouseover='this.src=button_resize_over.src' onmouseout='this.src=button_resize.src' ondragstart="return false;" style="cursor: se-resize;" id="rb" src="" /> <script>document.getElementById('rb').src = document.getElementById('button_resize').src</script>
  </tr>
 </table>
</div>
<div style='cursor: crosshair; height: 0px; width: 0px; border: none; position: absolute;' id='limit_div_0_0'
    onmousemove="slideCanvas(event,0);"
    onmouseup="releaseCanvas(event,0);" ondragover="onDragOver(event);" ondrop="onDrop(event,0);">
</div>
<div id='zoom_div_0' style='height: 0px; width: 0px; border: 1px dashed grey; position: absolute; left: 0px; top: 0px;'
 onmousemove="slideCanvas(event,0);" onmouseup="releaseCanvas(event,0);">
</div>
</span>
</span>
<div style='position: absolute; display: none; border: 2px dashed grey;' id='resize_div'></div>

<script type="text/javascript">
    var sockets = new Array();
     // holds the collection of sockets for this page
    var contexts = new Array();
     // holds the contexts for this page
    var canvii = new Array();
    var last_frames = new Array();
    var last_id = 0;
    var base_span = document.getElementById('plot_canvas_0');
    var ztop = 0;
    var base_port = 4567;
    var management_socket = null;
    var canvas_errors = new Array();

    var free_frames = new Array();
    free_frames[0] = 0;
    free_frames[1] = 0;
    free_frames[2] = 0;
    free_frames[3] = 0;

    function plot_if_possible(port) {
     // are we plotting this frame ?
     for (var i=0; i<last_id+1;i++) { if (free_frames[i] == port) return;}
     // if we have an empty / stopped canvas then plot this up
     for (var i=0; i<last_id+1;i++) {
      if (free_frames[i] == 0) { 
        start_plotting(i, port);
        break;
       }
     }
    }

    var layouts = new Array();
    layouts[1] = [10, 105, 800, 600];
    layouts[2] = [10, 105, 640, 480, 660, 105, 640, 480];
    layouts[4] = [10, 105, 453, 340, 477, 105, 453, 340, 10, 474, 453, 340, 477, 474, 453, 340];

    
    function set_layout(t) {
     for (var i=last_id;i<t-1;i++) {
      add_canvas();
     } // make sure we have enough canvii lurking
     try {
      for (var i=0;i<thumbnails.thumbnail_ports.length;i++) { plot_if_possible(thumbnails.thumbnail_ports[i]);}
     } catch (err) {canvas_errors.push("set_layout(" + t + "): " + err);}
      // see if this gives use any more plotting opportunities
     for (var i=t;i<last_id+1;i++) {
      document.getElementById("plot_canvas_" + i).style.display = "none";
      stop_plotting(i);
     } // make sure non used canvii are inactive and not visible
     for (var i=0;i<t;i++) {
      var plc = document.getElementById("plot_canvas_" + i);
      plc.style.left = layouts[t][i*4] + "px";
      plc.style.top = layouts[t][i*4 + 1] + "px";
      do_resize(i,layouts[t][i*4 + 2],layouts[t][i*4 + 3]);
      plc.style.display = "inline";
       // in case we have any data for this canvas..
     }
     // see if this gives use any more plotting opportunities
    }

    function maximise(id) {
     var w = document.width * 0.98 - document.getElementById('plot_canvas_' + id).offsetLeft; 
     var h = document.height * 0.98 - document.getElementById('plot_canvas_' + id).offsetTop - 20;
     resize = id;
     do_resize(id, w, h);
     resize = -1;
    }


    function add_canvas() {
     last_id += 1;
     var sp = document.getElementById('plot_canvas_0').cloneNode();
     sp.id = "plot_canvas_" + last_id;
     var t = document.getElementById('plot_canvas_0').innerHTML;
     t = t.replace(/0\)/g,last_id + ")");
     t = t.replace(/\_0\"/g,"_" + last_id + '"');
     sp.innerHTML = t
     document.getElementById('plot_canvii').appendChild(sp); 
      // position new plot
     var pc = document.getElementById('plot_canvas_' + last_id);
     if (last_id % 2 == 0) {
      pc.style.top = 100 + ((last_id / 2) * 530) + "px";
      pc.style.left = "10px";
     }
     else {
      pc.style.top = 100 + (((last_id - 1)/2) * 530) + "px";
      pc.style.left = "660px";
     }
     document.getElementById('status_' + last_id).innerText = "Disconnected"
     canvii[last_id] = document.getElementById('canvas_' + last_id);
     contexts[last_id] = canvii[last_id].getContext('2d');
     if (!(last_id in ldiv)) {
      ldiv[last_id] = new Array();
      ldiv[last_id][0] = document.getElementById('limit_div_0_' + last_id);
      ldiv[last_id][0].addEventListener('mousedown', function (e) {wrapClickCanvas(e,this);}, false);
      frame_counter[last_id] = 0;
      frame_start[last_id] = 0;
      cursor_info[last_id] = 0;
     }
     zdiv[last_id] = document.getElementById('zoom_div_' + last_id);
    }

    var ax_bb = new Array();
    cursor_info = new Array();
    cursor_info[0] = 0;
    frame_counter = new Array();
    frame_counter[0] = 0;
    frame_start = new Array();
    frame_start[0] = 0;

    function change_cursor_info(id) {
     document.getElementById('cursor_info_' + id).innerText = "";
     cursor_info[id] += 1;
     if (cursor_info[id] > 1) cursor_info[id] = 0;
    }

    function exec_user_cmd(id, cmd_str) {
     var ret_str = "";
     try {
      ret_str = eval(cmd_str);
     } catch(err) { ret_str = "user command failed: " + err;}
     if (id in sockets) {
      try {
       sockets[id].send("<user_cmd_ret args='" + ret_str + "'>");
      } catch (err) { alert('error returning output of user cmd:' + err); }
     }
    }

    function draw_frame(id) {
     try {
      if (frame_counter[id] == 0) { frame_start[id] = new Date().getTime();}
      var c = contexts[id];
      //if (id in sockets) ldiv[id][0].style.display = "inline";
      //else ldiv[id][0].style.display = "none";
       // when in client mode we cannot zoom anyway...
      for (var i=0; i < ldiv[id].length; i++) ldiv[id][i].style.display= "none";
       // hide any existing limit divs...
      ax_bb = new Array();
      eval(last_frames[id]); 
      frame_header();
       // execute the header. This will perform initial setup that is required (such as images) and then run frame_body..
       // we need a zoom limit div per axes
      for (var i=0; i < ax_bb.length; i++) {
       if (!(i in ldiv[id])) {
        var nid = 'limit_div_' + i + '_' + id;  
        var ndiv = ldiv[id][0].cloneNode();
        ndiv.id = nid;
         // fix the id
        ndiv.removeEventListener('mousedown');
        ndiv.addEventListener('mousedown', function (e) {wrapClickCanvas(e,this);}, false);
        document.getElementById('plot_canvas_' + id).appendChild(ndiv);
        ldiv[id][i] = document.getElementById('limit_div_' + i + '_' + id); 
       } // we need a limit div for this axes
       ldiv[id][i].style.display = "inline";
       ldiv[id][i].style.left = canvii[id].offsetLeft + ax_bb[i][0] + "px";
       ldiv[id][i].style.top = canvii[id].offsetTop + ax_bb[i][1] + "px";
       ldiv[id][i].style.width = ax_bb[i][4] - ax_bb[i][0] + "px";
       ldiv[id][i].style.height = ax_bb[i][3] - ax_bb[i][1] + "px";
       frame_counter[id] += 1;
       if (frame_counter[id] > 30) { 
        fps = (frame_counter[0] / (new Date().getTime() - frame_start[id]) * 1000);
        if (cursor_info[id] == 1) document.getElementById('cursor_info_' + id).innerText = "FPS:" + fps;
        frame_counter[id] = 0;
       }
      }
     } catch (err) {canvas_errors.push("draw_frame(" + id + "): " + err);}
    }

    var last_manage = "";
    var server_port = <!--server_port-->;
    function connect_manager() {
     if (management_socket) management_socket.close()
     management_socket = new WebSocket('ws://<!--server_ip-->:' + (server_port+1) + '/thumbs');
     management_socket.onmessage = function(e) { 
      last_manage = e.data;
      eval(e.data);
     } // end of function(e)
     management_socket.onopen = function(e) {
      management_socket.send("update");
      <!--requested_layout-->
     }
    }

    function update_thumbnails() {
     try {
      document.getElementById('thumbnails').src = "http://<!--server_ip-->:" + server_port + "/thumbs";
     } catch (err) { canvas_errors.push("failed to issue update thumbnails"); }
    }

    function start_plotting(id, port) {
     free_frames[id] = port;
     //if (!(id in sockets)) {
      sockets[id] = new WebSocket('ws://<!--server_ip-->:' + port + '/do');
      document.getElementById('status_' + id).innerText = "Connecting to port " + port + "..."
      sockets[id].onmessage = function(e) { 
       document.getElementById('status_' + id).innerText = "Connected"
       if (e.data.indexOf("/*exec_user_cmd*/") == 0) {
        exec_user_cmd(id, e.data);
       } else {
        last_frames[id] = e.data;
        draw_frame(id);
       }
      } // end of function(e)
     //} // end of if id in sockets
    }

    var allow_resize = true;
    function resize_canvas(id, width, height) {
     if (allow_resize) {
      if (id >= 0) {
       canvii[id].width = width; 
       document.getElementById("button_menu_" + id).style.width = width + "px";
       canvii[id].height = height;
      }
     }
    }

    function stop_plotting(id) { 
     free_frames[id] = 0;
     if (id in sockets) {
      sockets[id].onmessage = function(e) {};
       // reset the handler so that the buffer behind this socket does not polute new plots
      sockets[id].close();
      sockets.pop(id);
      last_frames[id] = "";
      document.getElementById('status_' + id).innerText = "Disconnected";
     }
    }

    // create the contexts for our canvii
    canvii[0] = document.getElementById('canvas_0');
    contexts[0] = canvii[0].getContext('2d');
    
    var zdiv = new Array();
    zdiv[0] = document.getElementById('zoom_div_0');
    var ldiv = new Array();
    ldiv[0] = new Array();
    ldiv[0][0] = document.getElementById('limit_div_0_0');
    ldiv[0][0].addEventListener('mousedown', function (e) {clickCanvas(e, 0, 0);}, false);
     // this style of event listener is an issue in Firefox 3.7. Will need to fix at some stage...

    var native_w = new Array();
    var native_h = new Array();
    var zdraw = -1;
    var resize = -1;
    var move = -1;
    var startX = 0;
    var startY = 0;
    var stopX = 0;
    var stopY = 0;
    var rStartX = 0;
    var rStartY = 0;
    var mStartX = 0;
    var mStartY = 0;

    var top_e = null;
    function wrapClickCanvas(e, ref) {
     var p = ref.id.split("_");
      // extract the figure and axes ids
     clickCanvas(e, p[3], p[2]);
    }

    function handle_user_event(arg_string, id) {
     if (id in sockets) {
      try {
       sockets[id].send("<user_event args='" + id + "," + arg_string + "'>");
      } catch (err) {}
     } 
    }

    function handle_click(e, id) {
     if (id in sockets) {
      try {
       var pc = document.getElementById('plot_canvas_' + id);
       sockets[id].send("<click args='" + (e.pageX - pc.offsetLeft) + "," + (canvii[id].clientHeight - (e.pageY - pc.offsetTop)) + "," + (e.button + 1) + "'>");
        // we need coords based on 0,0 in bottom left corner...
      } catch (err) {}
     }
    }

    var top_e = null;
    function clickCanvas(e,id,axes) {
     //alert("Clicked canvas for id " + id + " and axes " + axes);
     if (!e) var e = window.event;
        // e.button: 0 is left, 1 is middle, 2 is right.
     if ((e.button == 0) && (e.shiftKey == false)) {
      top_e = e;
      if (id > -1) zoom_canvas_id = id;
      var cnv = document.getElementById('plot_canvas_' + id);
      zdraw = axes;
      zdiv[id].style.width = 0;
      zdiv[id].style.height = 0;
      zdiv[id].style.top = (e.pageY - (cnv.offsetTop + cnv.offsetParent.offsetTop)) + "px";
      zdiv[id].style.left = (e.pageX - (cnv.offsetLeft + cnv.offsetParent.offsetLeft)) + "px";
      zdiv[id].style.display = "inline";
       // position the start of the zoom reticule
     }
     else {
      //zoomButtonDown = true;
     }
     startX = e.pageX;
     startY = e.pageY;
     pause = true;
     return false;
    }

    function clickMove(e,id) {
     move = id;
     mStartY = (e.pageY - document.getElementById('plot_canvas_' + id).offsetTop);
     mStartX = (e.pageX - document.getElementById('plot_canvas_' + id).offsetLeft);
     ztop += 1;
     document.getElementById('plot_canvas_' + id).style.setProperty('z-index',ztop);
    }

    function clickSize(e,id) {
     var cr = document.getElementById('resize_div');
     resize = id;
     rStartX = e.pageX;
     rStartY = e.pageY;
     document.getElementById('status_0').innerText = "Click size at " + rStartX + "," + rStartY;
     var pcs = document.getElementById('plot_canvas_' + id);
     cr.style.top = pcs.style.top;
     cr.style.left = pcs.style.left;
     cr.style.width = (pcs.clientWidth  - 2) + "px";
     cr.style.height = (pcs.clientHeight  - 4) + "px";
     cr.style.display = "inline";
     return false;
    }

    function slideSize(e) {
     if (resize > -1) {
      var cr = document.getElementById('resize_div');
      //resize_canvas(resize, (cw + (e.pageX - rStartX)), (ch + (e.pageY - rStartY)));
      cr.style.width = (e.pageX - cr.offsetLeft) + "px";
      cr.style.height = (e.pageY - cr.offsetTop) + "px";
      document.getElementById('status_0').innerText = "Slide size to " + (e.pageX - rStartX) + "," + (e.pageY - rStartY);
     }
     else if (move > -1) {
      var plc = document.getElementById('plot_canvas_' + move);
      plc.style.top = (e.pageY - mStartY) + "px";
      plc.style.left = (e.pageX - mStartX) + "px";
     }
     return false;
    }

    function do_resize(id, w, h) {
     if (id in sockets) {
      try {
       sockets[id].send("<resize args='" + w + "," + h + "'>");
      } catch (err) {}
     }
     else {
      var xScale = w / native_w[id];
      var yScale = h / native_h[id];
      resize_canvas(id, w, h);
       // no figure active for this canvas so do a purely client side resize
      allow_resize = false;
      canvii[id].width = canvii[id].width;
       // clear the canvas and reset scale factor before client size redraw
      document.getElementById('status_' + id).innerText = "Client side resize mode";
      contexts[id].scale(xScale, yScale);
       // needs to be done after resize_canvas, but this loses canvii[id].width/height so extra vars needed
      draw_frame(id);
       // the frame we draw may contain resize commands. Ignore these in client only mode hence the bracketing allow_resize directives.
      allow_resize = true;
     }
    }

    function outSize() {
     if (resize > -1) {
      var cr = document.getElementById('resize_div');
      do_resize(resize, cr.clientWidth, cr.clientHeight - 20);
      cr.style.display = "none";
     }
     resize = -1;
     move = -1;
     zdraw = -1;
      // make sure we kill everything on mouse up
    }

    function close_plot(id) {
     sockets[id].send("<close args=''>");
     stop_plotting(id);
     canvii[id].width = canvii[id].width;
    }

    function go_home(id) {
     sockets[id].send("<home args=''>");
    }

    var zoom_canvas_id = 0;

    function onDragOver(e) {
     if (e.preventDefault) e.preventDefault();
     this.className = 'over';
     e.dataTransfer.dropEffect = 'copy';
     return false;
    }

    function onDrop(e, id)
    {
     if (e.preventDefault) { e.preventDefault(); }
      // stop Firefox triggering a page change event
     top_e = e;
     var dt= e.dataTransfer;
     console.log(dt);
     var el_id = e.dataTransfer.getData('Text');
     stop_plotting(id);
     start_plotting(id, thumbnails.thumbnail_ports[el_id]);
     return false;
    }

    function zoom_in(id, axes) {
     var atop = 0;
     var aleft = 0;
     if (document.getElementById("anchor_div") != null) {
      an = document.getElementById("anchor_div");
      atop = an.offsetTop;
      aleft = an.offsetLeft;
     }
        
     var plc = document.getElementById("plot_canvas_" + id);
     var zoom_coords = axes + "," + (startX - (plc.offsetLeft + aleft)) + "," + (canvii[id].height - (stopY - (plc.offsetTop + atop))) + "," + (stopX - (plc.offsetLeft + aleft)) + "," + (canvii[id].height - (startY - (plc.offsetTop + atop)));
     sockets[id].send("<zoom args='" + zoom_coords + "'>");
     startX=stopX=startY=stopY=0;
     zdiv[id].style.width="0px";
     zdiv[id].style.height="0px";
     zdiv[id].style.display = "none";
    }

    function releaseCanvas(e,id) {
     if (!e) var e = window.event;
     top_e = e;
     stopX = e.pageX;
     stopY = e.pageY;
     if (zdraw > -1 && ((stopX-startX)>5) && ((stopY-startY)>5)) { zoom_in(zoom_canvas_id, zdraw);}
     else {
      // not in zdraw (or zoomed areas less than 5x5) so normal click
      handle_click(e,id);
      zdiv[id].style.display = "none";
     }
     zdraw = -1;
     pause = false;
     return false;
    }

    function slideCanvas(e,id,axes) {
     if (!e) var e = window.event;
     if (zdraw > -1)  {
      zdiv[id].style.width = e.pageX - startX + "px";
      zdiv[id].style.height = e.pageY - startY + "px";
     }
     else {
     }
     if (cursor_info[id] == 0) document.getElementById('cursor_info_' + id).innerText = "Cursor at: " + e.pageX + "," + e.pageY;
     return false;
    }
    document.captureEvents(Event.MOUSEMOVE)
    document.onmousemove = slideSize;
</script>
</body>
</html>
"""



# Copyright 2010, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Web Socket handshaking.

Note: request.connection.write/read are used in this module, even though
mod_python document says that they should be used only in connection handlers.
Unfortunately, we have no other options. For example, request.write/read are
not suitable because they don't allow direct raw bytes writing/reading.
"""


DEFAULT_WEB_SOCKET_PORT = 80
DEFAULT_WEB_SOCKET_SECURE_PORT = 443
WEB_SOCKET_SCHEME = 'ws'
WEB_SOCKET_SECURE_SCHEME = 'wss'


class HandshakeError(Exception):
    """Exception in Web Socket Handshake."""

    pass


def default_port(is_secure):
    if is_secure:
        return DEFAULT_WEB_SOCKET_SECURE_PORT
    else:
        return DEFAULT_WEB_SOCKET_PORT


def validate_protocol(protocol):
    """Validate WebSocket-Protocol string."""

    if not protocol:
        raise HandshakeError('Invalid WebSocket-Protocol: empty')
    for c in protocol:
        if not 0x20 <= ord(c) <= 0x7e:
            raise HandshakeError('Illegal character in protocol: %r' % c)


def parse_host_header(request):
    fields = request.headers_in['Host'].split(':', 1)
    if len(fields) == 1:
        return fields[0], default_port(request.is_https())
    try:
        return fields[0], int(fields[1])
    except ValueError, e:
        raise HandshakeError('Invalid port number format: %r' % e)


def build_location(request):
    """Build WebSocket location for request."""
    location_parts = []
    if request.is_https():
        location_parts.append(WEB_SOCKET_SECURE_SCHEME)
    else:
        location_parts.append(WEB_SOCKET_SCHEME)
    location_parts.append('://')
    host, port = parse_host_header(request)
    connection_port = request.connection.local_addr[1]
    if port != connection_port:
        raise HandshakeError('Header/connection port mismatch: %d/%d' %
                             (port, connection_port))
    location_parts.append(host)
    if (port != default_port(request.is_https())):
        location_parts.append(':')
        location_parts.append(str(port))
    location_parts.append(request.uri)
    return ''.join(location_parts)



# vi:sts=4 sw=4 et



"""A base for handling management of the h5 canvas backend.

Its jobs are as follows:
- Provide a standardised base port for clients to connect to
- Serve up the html wrapper page
- Provide a list of currently available plots (perhaps with a thumbnail)
- Manage the list of plots as time goes by

Simon Ratcliffe (sratcliffe@ska.ac.za)
Ludwig Schwardt (ludwig@ska.ac.za)

Copyright (c) 2010-2011, SKA South Africa
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of SKA South Africa nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

import BaseHTTPServer
import simple_server
import msgutil
import base_page
import thread
import sys
import re
import socket
import time
try:
    import netifaces
except:
    netifaces = None

class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    base_html = base_page.base_html
    base_html_decoration = base_page.base_html_decoration
    base_html_canvii = base_page.base_html_canvii
    thumb_html = base_page.thumb_html
    thumb_inner = base_page.thumb_inner
    h5m = None
    server_ip = ""
    server_port = ""
    custom_content = None
    def do_GET(self):
        match = re.compile("\/(\d*)$").match(self.path)
        ports = self.h5m._figures.keys()
        ports.sort()
        if match is not None:
            req_layout = match.groups()[0]
            for port in ports:
                canvas = self.h5m._figures[port]
            req_layout = (req_layout == '' and "" or "set_layout(" + str(req_layout) + ");")
            bh = self.base_html + self.base_html_decoration + self.base_html_canvii
            self.wfile.write(bh.replace('<!--requested_layout-->',req_layout).replace('<!--server_ip-->',self.server_ip).replace('<!--server_port-->',self.server_port).replace('<!--canvas_top-->','105').replace('<!--canvas_left-->','10').replace('<!--canvas_position-->','absolute'))
        elif self.path.startswith("/figure"):
            try:
                fig_no = int(self.path[7:]) - 1
            except ValueError:
                fig_no = 0
            if fig_no < 0: fig_no = 0
             # get the first figure by default
            try:
                port = ports[fig_no]
                custom_content = self.h5m._figures[port]._custom_content
                bh = self.base_html + self.base_html_canvii
                req_layout = "plot_if_possible(" + str(port)  + ");"
                content = bh.replace('<!--requested_layout-->',req_layout).replace('<!--server_ip-->',self.server_ip).replace('<!--server_port-->',self.server_port).replace('<!--canvas_top-->','10').replace('<!--canvas_left-->','10').replace('<!--canvas_position-->','absolute')
                if custom_content is not None:
                    content = custom_content.replace("<!--figure-->", content)
                self.wfile.write(content)

            except IndexError:
                self.wfile.write("Invalid Figure number (" + str(fig_no+1) + ") specified.")
        elif self.path == "/thumbs":
             # for each figure, create a thumbnail snippet and slipstream the js for the preview
            figure_count = 0
            thumbs = ""
            for port in ports:
                canvas = self.h5m._figures[port]
                #print "Found a figure at port",port
                t = self.thumb_inner.replace("<id>",str(figure_count))
                t = t.replace("<!--thumbnail_port-->",str(port))
                t = t.replace("<!--width-->",str(canvas._width)).replace("<!--height-->",str(canvas._height))
                frame = str(canvas._frame).replace("\n","").replace(";c.",";c_t_" + str(figure_count) + ".").replace("{ c", "{ c_t_" + str(figure_count))
                header = str(canvas._header).replace("\n","")
                if frame.startswith("c."): frame = "c_t_" + str(figure_count) + frame[1:]
                thumbs += t.replace('<!--thumbnail_content-->',header + frame) + "\n"
                #print t.replace('<!--thumbnail_content-->',str(canvas._frame).replace("\n","")) + "\n"
                figure_count += 1
             # insert thumbnail code into base page 
            self.wfile.write(self.thumb_html.replace("<!--thumbnail_body-->",thumbs))
        else:
            self.wfile.write("Not found...")

class H5Manager(object):
    """An H5 Canvas Manager.
    
    Parameters
    ----------
    port : integer
        The base port on which to serve the managers web interface
    """
    def __init__(self, base_port, limit):
        self.ip = self._external_ip()
         # find the next available port in multiples of 100 from base port
        self._figures = {}
        for x in range(limit):
            port_to_try = base_port + x*100
            try:
                self._server = BaseHTTPServer.HTTPServer(('', port_to_try), RequestHandler)
                self._thread = thread.start_new_thread(self._server.serve_forever, ())
                self._wsserver = simple_server.WebSocketServer(('', port_to_try+1), self.management_request, simple_server.WebSocketRequestHandler)
                self._wsthread = thread.start_new_thread(self._wsserver.serve_forever, ())
                break
            except Exception, e:
                if x == limit - 1:
                    print "Tried to find an available port pair from %i to %i, but none seemed available. You can only spawn %i python mplh5 instances on this machine.\nThis limit can be changed in init.py." % (base_port, base_port + (x-1)*100, limit)
                    sys.exit(1)
                else:
                    print "Failed to start management servers on ports (%i, %i). Trying another pair..." % (port_to_try, port_to_try+1)
                    time.sleep(0.05)
         # we have a port :)
        self.port = port_to_try
        RequestHandler.h5m = self
        RequestHandler.server_ip = self.ip
        RequestHandler.server_port = str(self.port)
        self.url = "http://%s:%i" % (self.ip, self.port)
        self._request_handlers = {}
        print "===================================================================================="
        print "Management interface active. Browse to %s to view plots." % self.url
        print "===================================================================================="
        sys.stdout.flush()

    def _external_ip(self, preferred_ifaces=('eth0', 'en0')):
        """Return the external IPv4 address of this machine.

        Attempts to use netifaces module if available, otherwise
        falls back to socket.

        Returns
        -------
        ip : str or None
            IPv4 address string (dotted quad). Returns None if
            ip address cannot be guessed.
        """
        if netifaces is None:
            ips = [socket.gethostbyname(socket.gethostname())]
        else:
            preferred_ips = []
            other_ips = []
            for iface in netifaces.interfaces():
                for addr in netifaces.ifaddresses(iface).get(netifaces.AF_INET, []):
                    if 'addr' in addr:
                        if iface in preferred_ifaces:
                            preferred_ips.append(addr['addr'])
                        else:
                            other_ips.append(addr['addr'])
            ips = preferred_ips + other_ips

        if ips:
            return ips[0]
        else:
            return "127.0.0.1"

    def management_request(self, request):
        self._request_handlers[request] = request.connection.remote_addr[0]
        while True:
            try:
                line = msgutil.receive_message(request).encode('utf-8')
                msgutil.send_message(request, "update_thumbnails();".decode('utf-8'))
            except Exception, e:
                #print "\nRemoving registered management handler",e
                if self._request_handlers.has_key(request): del self._request_handlers[request]
                return

    def tell(self):
        recipients = ""
        for r in self._request_handlers.keys():
            try:
                recipients += str(r.connection.remote_addr[0]) + " "
                msgutil.send_message(r, "update_thumbnails();".decode('utf-8'))
            except AttributeError:
                print "Connection",r.connection.remote_addr[0],"has gone. Closing..."
                del self._request_handlers[request]


    def add_figure(self, port, canvas):
        """Add a figure to the manager"""
        self._figures[port] = canvas
        self.tell()

    def remove_figure(self, port):
        """Remove a figure from the manager"""
        self._figures.pop(port)
        self.tell()

    def handle_base(self):
        print "Request for base page..."

    def parse_web_cmd(self, s):
        action = s[1:s.find(" ")]
        args = s[s.find("args='")+6:-2].split(",")
        method = getattr(self, "handle_%s" % action)
        if method:
            method(*args)
        else:
            self.handle_base()

    def on_request(self, request):
        while True:
            try:
                line = msgutil.receive_message(request).encode('utf-8')
                print "Received:",line
                self.parse_web_cmd(line)
            except Exception, e:
                print "\nCaught exception. Removing registered handler",e
                return



# Modifications 2010 by Simon Ratcliffe (sratcliffe@ska.ac.za)
#
# Original copyright notice below:
#
# Copyright 2009, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Simplistic WebSocket server.
"""

import BaseHTTPServer
import CGIHTTPServer
import SimpleHTTPServer
import SocketServer
import logging
import logging.handlers
import optparse
import os
import re
import socket
import sys

import traceback
import handshake
import memorizingfile
import threading
import select

# 1024 is practically large enough to contain WebSocket handshake lines.
_MAX_MEMORIZED_LINES = 1024

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
handler = logging.StreamHandler()
formatter = logging.Formatter("[%(asctime)s] [%(levelname)s] %(name)s: %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)

class _StandaloneConnection(object):
    """Mimic mod_python mp_conn."""

    def __init__(self, request_handler):
        """Construct an instance.

        Args:
            request_handler: A WebSocketRequestHandler instance.
        """
        self._request_handler = request_handler

    def get_local_addr(self):
        """Getter to mimic mp_conn.local_addr."""
        return (self._request_handler.server.server_name,
                self._request_handler.server.server_port)
    local_addr = property(get_local_addr)

    def get_remote_addr(self):
        """Getter to mimic mp_conn.remote_addr.

        Setting the property in __init__ won't work because the request
        handler is not initialized yet there."""
        return self._request_handler.client_address
    remote_addr = property(get_remote_addr)

    def write(self, data):
        """Mimic mp_conn.write()."""
        return self._request_handler.wfile.write(data)

    def read(self, length):
        """Mimic mp_conn.read()."""
        return self._request_handler.rfile.read(length)

    def get_memorized_lines(self):
        """Get memorized lines."""
        return self._request_handler.rfile.get_memorized_lines()


class _StandaloneRequest(object):
    """Mimic mod_python request."""

    def __init__(self, request_handler, use_tls):
        """Construct an instance.

        Args:
            request_handler: A WebSocketRequestHandler instance.
        """
        self._request_handler = request_handler
        self.connection = _StandaloneConnection(request_handler)
        self._use_tls = use_tls

    def get_uri(self):
        """Getter to mimic request.uri."""
        return self._request_handler.path
    uri = property(get_uri)

    def get_method(self):
        """Getter to mimic request.method."""
        return self._request_handler.command
    method = property(get_method)

    def get_headers_in(self):
        """Getter to mimic request.headers_in."""
        return self._request_handler.headers
    headers_in = property(get_headers_in)

    def is_https(self):
        """Mimic request.is_https()."""
        return self._use_tls


class WebSocketServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    """HTTPServer specialized for Web Socket."""

    SocketServer.ThreadingMixIn.daemon_threads = True

    def __init__(self, server_address, handler, RequestHandlerClass):
        """Override SocketServer.BaseServer.__init__."""

        SocketServer.BaseServer.__init__(
                self, server_address, RequestHandlerClass)
        self._handler = handler
        self.socket = self._create_socket()
        self.server_bind()
        self.server_activate()
         # python 2.5 fixes
        self.__is_shut_down = threading.Event()
        self.__serving = False

     # from python 2.6 to handle python 2.5 :)
    def serve_forever(self, poll_interval=0.5):
        """Handle one request at a time until shutdown.

        Polls for shutdown every poll_interval seconds. Ignores
        self.timeout. If you need to do periodic tasks, do them in
        another thread.
        """
        self.__serving = True
        self.__is_shut_down.clear()
        while self.__serving:
            r, w, e = select.select([self], [], [], poll_interval)
            if r:
                self._handle_request_noblock()
        self.__is_shut_down.set()

    def shutdown(self):
        """Stops the serve_forever loop.

        Blocks until the loop has finished. This must be called while
        serve_forever() is running in another thread, or it will
        deadlock.
        """
        self.__serving = False
        self.__is_shut_down.wait()

    def handle_request(self):
        """Handle one request, possibly blocking.

        Respects self.timeout.
        """
        # Support people who used socket.settimeout() to escape
        # handle_request before self.timeout was available.
        timeout = self.socket.gettimeout()
        if timeout is None:
            timeout = self.timeout
        elif self.timeout is not None:
            timeout = min(timeout, self.timeout)
        fd_sets = select.select([self], [], [], timeout)
        if not fd_sets[0]:
            self.handle_timeout()
            return
        self._handle_request_noblock()

    def _handle_request_noblock(self):
        """Handle one request, without blocking.

        I assume that select.select has returned that the socket is
        readable before this function was called, so there should be
        no risk of blocking in get_request().
        """
        try:
            request, client_address = self.get_request()
        except socket.error:
            return
        if self.verify_request(request, client_address):
            try:
                self.process_request(request, client_address)
            except:
                self.handle_error(request, client_address)
                self.close_request(request)

    def handle_timeout(self):
        """Called if no new request arrives within self.timeout.

        Overridden by ForkingMixIn.
        """
        pass

    def _create_socket(self):
        socket_ = socket.socket(self.address_family, self.socket_type)
        return socket_

    def finish_request(self, request, client_address):
        self.RequestHandlerClass(request, client_address, self, handler=self._handler)

    def handle_error(self, rquest, client_address):
        """Override SocketServer.handle_error."""

        logging.error(
            ('Exception in processing request from: %r' % (client_address,)) +
            '\n' + traceback.format_exc())
        # Note: client_address is a tuple. To match it against %r, we need the
        # trailing comma.


class WebSocketRequestHandler(CGIHTTPServer.CGIHTTPRequestHandler):
    """CGIHTTPRequestHandler specialized for Web Socket."""

    def setup(self):
        """Override SocketServer.StreamRequestHandler.setup."""

        self.connection = self.request
        self.rfile = memorizingfile.MemorizingFile(
                socket._fileobject(self.request, 'rb', self.rbufsize),
                max_memorized_lines=1024)
        self.wfile = socket._fileobject(self.request, 'wb', self.wbufsize)

    def __init__(self, *args, **keywords):
        self._request = _StandaloneRequest(
                self, False)
        self._handler = keywords.pop('handler') #WebSocketRequestHandler._handlers
        self._handshaker = handshake.Handshaker(self._request, allowDraft75=True, strict=False)
        CGIHTTPServer.CGIHTTPRequestHandler.__init__(
                self, *args, **keywords)

    def parse_request(self):
        """Override BaseHTTPServer.BaseHTTPRequestHandler.parse_request.

        Return True to continue processing for HTTP(S), False otherwise.
        """
        result = CGIHTTPServer.CGIHTTPRequestHandler.parse_request(self)
        #print "CGI Result: ",result
        if result:
            try:
                self._handshaker.do_handshake()
                self._handler(self._request)
                return False
            except handshake.HandshakeError, e:
                # Handshake for ws(s) failed. Assume http(s).
                logging.info('mod_pywebsocket: %s' % e)
                return True
            except Exception, e:
                logging.warning('mod_pywebsocket: %s' % e)
                logging.info('mod_pywebsocket: %s' % traceback.format_exc())
                return False
        return result

    def log_request(self, code='-', size='-'):
        """Override BaseHTTPServer.log_request."""

        logging.info('"%s" %s %s',
                     self.requestline, str(code), str(size))

    def log_error(self, *args):
        """Override BaseHTTPServer.log_error."""

        # Despite the name, this method is for warnings than for errors.
        # For example, HTTP status code is logged by this method.
        logging.warn('%s - %s' % (self.address_string(), (args[0] % args[1:])))

    def is_cgi(self):
        return False

# vi:sts=4 sw=4 et



#!/usr/bin/python
"""Used for testing image show."""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time

fig = figure()
ax = fig.add_subplot(111)
img = standard_normal((50,100))
image = ax.imshow(img,interpolation='nearest',animated=False,label="blah")
xlabel('time (s)')
ylabel('voltage (mV)')
title('Another test')
show(block=False)
 # show the figure manager but don't block script execution so animation works..
print "Animating... Ctrl-C to stop"
while True:
    img = standard_normal((50,100))
    image.set_data(img)
    fig.canvas.draw()
    time.sleep(0.1)



#!/usr/bin/python

# The MPLH5Canvas test suite.
#
# Basically this runs against a directory of matplotlib examples (try examples/api or examples/pylab in the matplotlib source tree)
# and produces a test.html file in ./output that when shown in a browser displays a MPLH5Canvas, PNG and SVG rendering for each
# example in the test set.
#
# This allows rapid fault finding through direct comparisons with known good backends...

from optparse import OptionParser
import os
import sys

parser = OptionParser()
parser.add_option("-d", "--dir", type="string", default=".",help="Specify the directory containing examples to be tested. [default=%default]")
parser.add_option("-f", "--file", type="string", default=None, help="Specify a single file to test. Overrides any directory specified. [default=%default]")
parser.add_option("-w","--wildcard", type="string", default="*", help="Match the filenames to use. e.g. * for all, image_ for all image demos. [default=%default]")
(options, args) = parser.parse_args()

exclusions = ['customize_rc.py','dannys_example.py','hexbin_demo.py','hexbin_demo2.py','pcolor_demo.py','pcolor_log.py','usetex_fonteffects.py','usetex_demo.py','usetex_baseline_test.py','to_numeric.py','tex_unicode_demo.py','tex_demo.py','scatter_profile.py','__init__.py','quadmesh_demo.py']
 # matplotlib examples that should be excluded as they bork things.
 #
 # dannys_example.py - uses tex - which borks matplotlib if you don't have text installed
 # usetext* , tex* - as above
 # hexbin_demo.py - seems to create an infinite length polygon...
 # pcolor_demo.py - what the heck is this rubbish. imshow ftw...
 # pcolor_log.py - see above
 # quadmesh_demo.py - very slow for now
 # scatter_profile.py - very slow

files = []
p = os.listdir(options.dir+"/")
if options.wildcard == "*": options.wildcard = ""
while p:
    x = p.pop()
    if x != sys.argv[0] and x.endswith(".py") and x not in exclusions and x.startswith(options.wildcard):
        files.append(x)

if options.file:
    options.dir = options.file[:options.file.rfind("/")]
    files = [options.file[options.file.rfind("/")+1:]]
    print "Dir:",options.dir,",Files:",files

if files == []:
    print "No .py files found in the specified directory (%s)" % options.dir
    sys.exit(0)

import matplotlib
import mplh5canvas.backend_h5canvas

mplh5canvas.backend_h5canvas._test = True
mplh5canvas.backend_h5canvas._quiet = True
# The 'use' command has to happen *before* pylab is touched 
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import savefig, gcf, switch_backend

html = "<html><head><script>function resize_canvas(id, w, h) { } var id=-1; var ax_bb = new Array(); var native_w = new Array(); var native_h = new Array();</script></head><body><table>"
html += "<tr><th>File<th>H5 Canvas<th>PNG<th>SVG</tr>"
thtml = "<html><head><body><table><tr><th>File<th>H5 Canvas (PNG from Chrome 4.0 OSX)<th>PNG</tr>"
files.sort()
for count, filename in enumerate(files):
    print "Running %s\n" % filename
    html += "<tr><th id='name_" + str(count) + "'>" + filename
    thtml += "<tr><th id='name_" + str(count) + "'>" + filename
    try:
        execfile(options.dir + "/" + filename)
        f = gcf()
        f.canvas.draw()
    except Exception, e:
        print "Failed to run script %s. (%s)" % (filename, str(e))
        html += "<td>Failed to execute script.<td>Failed to execute script.<td>Failed to execute script.</tr>"
        thtml += "<td>Failed to execute script.<td>Failed to execute script.<td>Failed to execute script.</tr>"
        continue
    f = gcf()
    f.canvas.draw(ctx_override="c_" + str(count))
    w, h = f.get_size_inches() * f.get_dpi()
    png_filename = filename[:-2] + "png"
    try:
        html += "<td><canvas width='%dpx' height='%dpx' id='canvas_%d'>" % (w, h, count,)
        html += "\n<script>var c_%d = document.getElementById('canvas_%d').getContext('2d');\n" % (count, count)
        html += f.canvas._frame_extra + "\n" + f.canvas._header + "\n" + f.canvas._frame
        html += "\nframe_header();\n"
        html += "\n</script></canvas>"
        thtml += "<td><img src='%s' width='%dpx' height='%dpx' />" % ("h5canvas_" + png_filename, w, h)
    except Exception, e:
        print "Failed to execute %s. (%s)" % (filename, str(e))
        html += "<td>Failed to create Canvas"
        thtml += "<td>Failed to create Canvas"

    try:
        f.canvas.print_png("./output/" + png_filename, dpi=f.dpi)
        html += "<td><img src='%s' width='%dpx' height='%dpx' />" % (png_filename, w, h)
        thtml += "<td><img src='%s' width='%dpx' height='%dpx' />" % (png_filename, w, h)
    except Exception, e:
        print "Failed to execute %s. (%s)" % (filename, str(e))
        html += "<td>Failed to create PNG"
        thtml += "<td>Failed to create PNG"

    try:
        svg_filename = filename[:-2] + "svg"
        f.canvas.print_svg("./output/" + svg_filename, dpi=f.dpi)
        html += "<td><img src='%s' width='%dpx' height='%dpx' />" % (svg_filename, w, h)
    except Exception, e:
        print "Failed to execute %s. (%s)" % (filename, str(e))
        html += "<td>Failed to create SVG"
    switch_backend('module://mplh5canvas.backend_h5canvas')
    html += "</tr>"
    thtml += "</tr>"

print "Finished processing files..."
html += "</table><script> var total_plots = " + str(count) + "; "
pi = """
function connect() {
  s = new WebSocket("ws://192.168.184.1:8123");
}

function put_images() {
  for (var i=0; i<total_plots+1;i++) { 
   try {
   s.send(document.getElementById("name_"+i).innerText.split(".py")[0] + ".png " + document.getElementById("canvas_"+i).toDataURL());
   } catch (err) {}
 }
}"""
html += pi +"</script><input type='button' onclick='put_images()' value='Put Images to server'>"
html += "<input type='button' onclick='connect()' value='Connect'></body></html>"
thtml += "</table></body></html>"
f = open("output/test.html","w")
f.write(html)
f.close()
f = open("output/test_rendered.html","w")
f.write(thtml)
f.close()



#!/usr/bin/env python
"""Trying out subplots."""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *

def f(t):
    s1 = cos(2*pi*t)
    e1 = exp(-t)
    return multiply(s1,e1)

t1 = arange(0.0, 5.0, 0.1)
t2 = arange(0.0, 5.0, 0.02)
t3 = arange(0.0, 2.0, 0.01)

subplot(211)
l = plot(t1, f(t1), 'bo', t2, f(t2), 'k--', markerfacecolor='green')
grid(True)
title('A tale of 2 subplots')
ylabel('Damped oscillation')

subplot(212)
plot(t3, cos(2*pi*t3), 'r.')
grid(True)
xlabel('time (s)')
ylabel('Undamped')
show()




#!/usr/bin/python
"""Plot embedded in HTML wrapper with custom user events..."""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time

sensor_list = ['enviro.wind_speed','enviro.wind_direction','enviro.ambient_temperature','enviro.humidity']

def user_cmd_ret(*args):
    """Handle any data returned from calls to canvas.send_cmd()"""
    print "Got return from user event:",args

def user_event(figure_id, *args):
    f = figure(int(figure_id)+1)
     # make the specified figure active for the rest of the calls in this method
    sensors = args[:-1]
    clf()
    xlabel('time (s)')
    ylabel('value')
    count = 1
    for sensor in sensors:
        t = arange(0, 100, 1)
        s = sin(count*pi*t/10) * 10
        plot(t,s,linewidth=1.0,label=sensor)
        count+=0.5
    legend()
    f.canvas.draw()
    f.canvas.send_cmd("alert('Server says: Plot updated...'); document.documentURI;")
     # send a command back to the browser on completion of the event
     # output of this command (in this case the documentURI is returned to the server if user_cmd_ret is set

# show a plot
title('No sensors selected')
f = gcf()
ax = gca()

# some sensors
sensor_select = "".join(['<option value="'+x+'">'+x+'</option>' for x in sensor_list])

f.canvas._user_event = user_event
 # register handler for client side javascript calls to user_event

f.canvas._user_cmd_ret = user_cmd_ret
 # register handler for any returns from send_cmd

html_wrap_file = open("./examples/monitor_plot.html")
cc = html_wrap_file.read().replace("<!--sensor-list-->",sensor_select)
 # read in a custom HTML wrapper and populate dynamic content

f.canvas._custom_content = cc
 # specify a custom HTML wrapper to use in place of default (thumbnail view)

html_wrap_file.close()
f.canvas.draw()

show(layout='figure1')



#!/usr/bin/python

"""Simple receiver to collect png images from the test page rendered from the various HTML5 canvii
on the test page. Run this if you want to collect rendered versions of the HTML canvii generated by the test scripts."""

from mplh5canvas import simple_server, msgutil
import thread
import time
import base64

def request(request):
    while True:
        try:
            line = msgutil.receive_message(request).encode('utf-8')
            (filename, b64) = line.split(" ")
             # should be filename<space>base64png
            try:
                s = base64.b64decode(b64[22:])
                 # skip space and data:image/png;base64, header
            except Exception,e:
                print "Base64 decoding failed. Maybe not an image...",e
                break
            if s[0:5] == '\x89PNG\r':
                print "Base64 decoding passed. Writing image to ",filename
                f = open("./output/h5canvas_" + filename, "w")
                f.write(s)
                f.close()
        except Exception,e:
            print "Failed...",e
            break

wsserver = simple_server.WebSocketServer(('', 8123), request, simple_server.WebSocketRequestHandler)
wsthread = thread.start_new_thread(wsserver.serve_forever, ())
while True:
    time.sleep(1)



#!/usr/bin/python
"""Simple static plot, mostly for testing zooming..."""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time


t = arange(0, 100, 1)
s = sin(2*pi*t/10) * 10
plot(t, s, linewidth=1.0)
xlabel('time (s)')
ylabel('voltage (mV)')
title('Frist Post')
f = gcf()
show()



# Copyright 2009, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Web Socket handshaking.

Note: request.connection.write/read are used in this module, even though
mod_python document says that they should be used only in connection handlers.
Unfortunately, we have no other options. For example, request.write/read are
not suitable because they don't allow direct raw bytes writing/reading.
"""


import logging
from md5 import md5
import re
import struct

from _base import HandshakeError
from _base import build_location
from _base import validate_protocol


_MANDATORY_HEADERS = [
    # key, expected value or None
    ['Upgrade', 'WebSocket'],
    ['Connection', 'Upgrade'],
]

def _hexify(s):
    return re.sub('.', lambda x: '%02x ' % ord(x.group(0)), s)

class Handshaker(object):
    """This class performs Web Socket handshake."""

    def __init__(self, request):
        """Construct an instance.

        Args:
            request: mod_python request.

        Handshaker will add attributes such as ws_resource in performing
        handshake.
        """

        self._logger = logging.getLogger("mod_pywebsocket.handshake")
        self._logger.setLevel(logging.ERROR)
        self._request = request

    def do_handshake(self):
        """Perform Web Socket Handshake."""

        # 5.1 Reading the client's opening handshake.
        self._check_header_lines()
        self._set_resource()
        self._set_protocol()
        self._set_location()
        self._set_origin()
        self._set_challenge_response()
        self._send_handshake()

    def _check_header_lines(self):
        # 5.1 1. The three character UTF-8 string "GET".
        # 5.1 2. A UTF-8-encoded U+0020 SPACE character (0x20 byte).
        if self._request.method != 'GET':
            raise HandshakeError('Method is not GET')
        # The expected field names, and the meaning of their corresponding
        # values, are as follows.
        #  |Upgrade| and |Connection|
        for key, expected_value in _MANDATORY_HEADERS:
            actual_value = self._request.headers_in.get(key)
            if not actual_value:
                raise HandshakeError('Header %s is not defined' % key)
            if expected_value:
                if actual_value != expected_value:
                    raise HandshakeError('Illegal value for header %s: %s' %
                                         (key, actual_value))

    def _set_resource(self):
        self._request.ws_resource = self._request.uri

    def _set_protocol(self):
        # |Sec-WebSocket-Protocol|
        protocol = self._request.headers_in.get('Sec-WebSocket-Protocol')
        if protocol is not None:
            validate_protocol(protocol)
        self._request.ws_protocol = protocol

    def _set_location(self):
        # |Host|
        host = self._request.headers_in.get('Host')
        if host is not None:
            self._request.ws_location = build_location(self._request)
        # TODO(ukai): check host is this host.

    def _set_origin(self):
        # |Origin|
        origin = self._request.headers_in['Origin']
        if origin is not None:
            self._request.ws_origin = origin

    def _set_challenge_response(self):
        # 5.2 4-8.
        self._request.ws_challenge = self._get_challenge()
        # 5.2 9. let /response/ be the MD5 finterprint of /challenge/
        self._request.ws_challenge_md5 = md5(
            self._request.ws_challenge).digest()
        self._logger.debug("challenge: %s" % _hexify(
            self._request.ws_challenge))
        self._logger.debug("response:  %s" % _hexify(
            self._request.ws_challenge_md5))

    def _get_key_value(self, key_field):
        key_value = self._request.headers_in.get(key_field)
        if key_value is None:
            self._logger.debug("no %s" % key_value)
            return None
        try:
            # 5.2 4. let /key-number_n/ be the digits (characters in the range
            # U+0030 DIGIT ZERO (0) to U+0039 DIGIT NINE (9)) in /key_n/,
            # interpreted as a base ten integer, ignoring all other characters
            # in /key_n/
            key_number = int(re.sub("\\D", "", key_value))
            # 5.2 5. let /spaces_n/ be the number of U+0020 SPACE characters
            # in /key_n/.
            spaces = re.subn(" ", "", key_value)[1]
            # 5.2 6. if /key-number_n/ is not an integral multiple of /spaces_n/
            # then abort the WebSocket connection.
            if key_number % spaces != 0:
                raise handshakeError('key_number %d is not an integral '
                                     'multiple of spaces %d' % (key_number,
                                                                spaces))
            # 5.2 7. let /part_n/ be /key_number_n/ divided by /spaces_n/.
            part = key_number / spaces
            self._logger.debug("%s: %s => %d / %d => %d" % (
                key_field, key_value, key_number, spaces, part))
            return part
        except:
            return None

    def _get_challenge(self):
        # 5.2 4-7.
        key1 = self._get_key_value('Sec-Websocket-Key1')
        if not key1:
            raise HandshakeError('Sec-WebSocket-Key1 not found')
        key2 = self._get_key_value('Sec-Websocket-Key2')
        if not key2:
            raise HandshakeError('Sec-WebSocket-Key2 not found')
        # 5.2 8. let /challenge/ be the concatenation of /part_1/,
        challenge = ""
        challenge += struct.pack("!I", key1)  # network byteorder int
        challenge += struct.pack("!I", key2)  # network byteorder int
        challenge += self._request.connection.read(8)
        return challenge

    def _send_handshake(self):
        # 5.2 10. send the following line.
        self._request.connection.write(
                'HTTP/1.1 101 WebSocket Protocol Handshake\r\n')
        # 5.2 11. send the following fields to the client.
        self._request.connection.write('Upgrade: WebSocket\r\n')
        self._request.connection.write('Connection: Upgrade\r\n')
        self._request.connection.write('Sec-WebSocket-Location: ')
        self._request.connection.write(self._request.ws_location)
        self._request.connection.write('\r\n')
        self._request.connection.write('Sec-WebSocket-Origin: ')
        self._request.connection.write(self._request.ws_origin)
        self._request.connection.write('\r\n')
        if self._request.ws_protocol:
            self._request.connection.write('Sec-WebSocket-Protocol: ')
            self._request.connection.write(self._request.ws_protocol)
            self._request.connection.write('\r\n')
        # 5.2 12. send two bytes 0x0D 0x0A.
        self._request.connection.write('\r\n')
        # 5.2 13. send /response/
        self._request.connection.write(self._request.ws_challenge_md5)


# vi:sts=4 sw=4 et



#!/usr/bin/python
"""Testbed for the animation functionality of the backend.

It basically produces an long series of frames that get animated on the client
browser side.

"""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time

def refresh_data(ax):
    t = arange(0.0 + count, 2.0 + count, 0.01)
    s = sin(2*pi*t)
    ax.lines[0].set_xdata(t)
    ax.lines[0].set_ydata(s)
    ax.set_xlim(t[0],t[-1])

t = arange(0.0, 2.0, 0.01)
s = sin(2*pi*t)
plot(t, s, linewidth=1.0)
xlabel('time (s)')
ylabel('voltage (mV)')
title('Frist Post')
f = gcf()
ax = f.gca()
count = 0

show(block=False)
 # show the figure manager but don't block script execution so animation works..
while True:
    refresh_data(ax)
    f.canvas.draw()
    count += 0.01
    time.sleep(0.5)



#!/usr/bin/python
"""Testbed for the animation functionality of the backend, with multiple figures.

It basically produces an long series of frames that get animated on the client
browser side, this time with two figures.

"""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time

def refresh_data(ax):
    t = arange(0.0 + count, 2.0 + count, 0.01)
    s = sin(2*pi*t)
    ax.lines[0].set_xdata(t)
    ax.lines[0].set_ydata(s)
    ax.set_xlim(t[0],t[-1])

t = arange(0.0, 2.0, 0.01)
s = sin(2*pi*t)
plot(t, s, linewidth=1.0)
xlabel('time (s)')
ylabel('voltage (mV)')
title('Frist Post')
f = gcf()
ax = f.gca()
count = 0

f2 = figure()
ax2 = f2.gca()
ax2.set_xlabel('IMDB rating')
ax2.set_ylabel('South African Connections')
ax2.set_title('Luds chart...')
ax2.plot(arange(0.0, 5 + count, 0.01), arange(0.0, 5 + count, 0.01))

show(block=False, layout=2)
 # show the figure manager but don't block script execution so animation works..
 # layout=2 overrides the default layout manager which only shows a single plot in the browser window

while True:
    refresh_data(ax)
    d = arange(0.0, 5 + count, 0.01)
    ax2.lines[0].set_xdata(d)
    ax2.lines[0].set_ydata(d)
    ax2.set_xlim(d[0],d[-1])
    ax2.set_ylim(d[0],d[-1])
    f.canvas.draw()
    f2.canvas.draw()
    count += 0.01
    time.sleep(1)



# Copyright 2010, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Web Socket handshaking.

Note: request.connection.write/read are used in this module, even though
mod_python document says that they should be used only in connection handlers.
Unfortunately, we have no other options. For example, request.write/read are
not suitable because they don't allow direct raw bytes writing/reading.
"""


import logging
import re

import draft75
import handshake
from _base import DEFAULT_WEB_SOCKET_PORT
from _base import DEFAULT_WEB_SOCKET_SECURE_PORT
from _base import WEB_SOCKET_SCHEME
from _base import WEB_SOCKET_SECURE_SCHEME
from _base import HandshakeError
from _base import validate_protocol


class Handshaker(object):
    """This class performs Web Socket handshake."""

    def __init__(self, request, allowDraft75=False, strict=False):
        """Construct an instance.

        Args:
            request: mod_python request.
            allowDraft75: allow draft 75 handshake protocol.
            strict: Strictly check handshake request in draft 75.
                Default: False. If True, request.connection must provide
                get_memorized_lines method.

        Handshaker will add attributes such as ws_resource in performing
        handshake.
        """

        self._logger = logging.getLogger("mod_pywebsocket.handshake")
        self._request = request
        self._strict = strict
        self._handshaker = handshake.Handshaker(request)
        self._fallbackHandshaker = None
        if allowDraft75:
            self._fallbackHandshaker = draft75.Handshaker(
                request, strict)

    def do_handshake(self):
        """Perform Web Socket Handshake."""

        try:
            self._handshaker.do_handshake()
        except HandshakeError, e:
            if self._fallbackHandshaker:
                self._logger.warning('fallback to old protocol')
                self._fallbackHandshaker.do_handshake()
                return
            raise e


# vi:sts=4 sw=4 et



# Modifications 2010 by Simon Ratcliffe (sratcliffe@ska.ac.za)
#
# Copyright 2009, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Message related utilities.

Note: request.connection.write/read are used in this module, even though
mod_python document says that they should be used only in connection handlers.
Unfortunately, we have no other options. For example, request.write/read are
not suitable because they don't allow direct raw bytes writing/reading.
"""


import Queue
import threading


class MsgUtilException(Exception):
    pass


def _read(request, length):
    bytes = request.connection.read(length)
    if not bytes:
        raise MsgUtilException(
                'Failed to receive message from %r' %
                        (request.connection.remote_addr,))
    return bytes


def _write(request, bytes):
    try:
        request.connection.write(bytes)
    except Exception, e:
        e.args = 'Failed to send message to %r: ' % (request.connection.remote_addr,) + str(e)
        raise


def send_message(request, message):
    """Send message.

    Args:
        request: mod_python request.
        message: unicode string to send.
    """

    _write(request, '\x00' + message.encode('utf-8') + '\xff')


def receive_message(request):
    """Receive a Web Socket frame and return its payload as unicode string.

    Args:
        request: mod_python request.
    """

    while True:
        # Read 1 byte.
        # mp_conn.read will block if no bytes are available.
        # Timeout is controlled by TimeOut directive of Apache.
        frame_type_str = _read(request, 1)
        frame_type = ord(frame_type_str[0])
        if (frame_type & 0x80) == 0x80:
            # The payload length is specified in the frame.
            # Read and discard.
            length = _payload_length(request)
            _receive_bytes(request, length)
        else:
            # The payload is delimited with \xff.
            bytes = _read_until(request, '\xff')
            # The Web Socket protocol section 4.4 specifies that invalid
            # characters must be replaced with U+fffd REPLACEMENT CHARACTER.
            message = bytes.decode('utf-8', 'replace')
            if frame_type == 0x00:
                return message
            # Discard data of other types.


def _payload_length(request):
    length = 0
    while True:
        b_str = _read(request, 1)
        b = ord(b_str[0])
        length = length * 128 + (b & 0x7f)
        if (b & 0x80) == 0:
            break
    return length


def _receive_bytes(request, length):
    bytes = []
    while length > 0:
        new_bytes = _read(request, length)
        bytes.append(new_bytes)
        length -= len(new_bytes)
    return ''.join(bytes)


def _read_until(request, delim_char):
    bytes = []
    while True:
        ch = _read(request, 1)
        if ch == delim_char:
            break
        bytes.append(ch)
    return ''.join(bytes)


class MessageReceiver(threading.Thread):
    """This class receives messages from the client.

    This class provides three ways to receive messages: blocking, non-blocking,
    and via callback. Callback has the highest precedence.

    Note: This class should not be used with the standalone server for wss
    because pyOpenSSL used by the server raises a fatal error if the socket
    is accessed from multiple threads.
    """
    def __init__(self, request, onmessage=None):
        """Construct an instance.

        Args:
            request: mod_python request.
            onmessage: a function to be called when a message is received.
                       May be None. If not None, the function is called on
                       another thread. In that case, MessageReceiver.receive
                       and MessageReceiver.receive_nowait are useless because
                       they will never return any messages.
        """
        threading.Thread.__init__(self)
        self._request = request
        self._queue = Queue.Queue()
        self._onmessage = onmessage
        self._stop_requested = False
        self.setDaemon(True)
        self.start()

    def run(self):
        while not self._stop_requested:
            message = receive_message(self._request)
            if self._onmessage:
                self._onmessage(message)
            else:
                self._queue.put(message)

    def receive(self):
        """ Receive a message from the channel, blocking.

        Returns:
            message as a unicode string.
        """
        return self._queue.get()

    def receive_nowait(self):
        """ Receive a message from the channel, non-blocking.

        Returns:
            message as a unicode string if available. None otherwise.
        """
        try:
            message = self._queue.get_nowait()
        except Queue.Empty:
            message = None
        return message

    def stop(self):
        """Request to stop this instance.

        The instance will be stopped after receiving the next message.
        This method may not be very useful, but there is no clean way
        in Python to forcefully stop a running thread.
        """
        self._stop_requested = True


class MessageSender(threading.Thread):
    """This class sends messages to the client.

    This class provides both synchronous and asynchronous ways to send
    messages.

    Note: This class should not be used with the standalone server for wss
    because pyOpenSSL used by the server raises a fatal error if the socket
    is accessed from multiple threads.
    """
    def __init__(self, request):
        """Construct an instance.

        Args:
            request: mod_python request.
        """
        threading.Thread.__init__(self)
        self._request = request
        self._queue = Queue.Queue()
        self.setDaemon(True)
        self.start()

    def run(self):
        while True:
            message, condition = self._queue.get()
            condition.acquire()
            send_message(self._request, message)
            condition.notify()
            condition.release()

    def send(self, message):
        """Send a message, blocking."""

        condition = threading.Condition()
        condition.acquire()
        self._queue.put((message, condition))
        condition.wait()

    def send_nowait(self, message):
        """Send a message, non-blocking."""

        self._queue.put((message, threading.Condition()))


# vi:sts=4 sw=4 et



#!/usr/bin/python
"""Plots of two different sizes!"""

import matplotlib
matplotlib.use('module://mplh5canvas.backend_h5canvas')
from pylab import *
import time

t = arange(0.0, 2.0, 0.01)
s = sin(2*pi*t)
f = figure(figsize=(10,6))
ax = f.gca()
ax.plot(t, s, linewidth=1.0)
ax.set_xlabel('time (s)')
ax.set_ylabel('voltage (mV)')
ax.set_title('Frist Post')
count = 0
f.show()

f2 = figure(figsize=(12,4))
ax2 = f2.gca()
ax2.set_xlabel('IMDB rating')
ax2.set_ylabel('South African Connections')
ax2.set_title('Luds chart...')
ax2.plot(arange(0.0, 5 + count, 0.01), arange(0.0, 5 + count, 0.01))

show()



# Copyright 2010, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Web Socket handshaking defined in draft-hixie-thewebsocketprotocol-75.

Note: request.connection.write/read are used in this module, even though
mod_python document says that they should be used only in connection handlers.
Unfortunately, we have no other options. For example, request.write/read are
not suitable because they don't allow direct raw bytes writing/reading.
"""


import re

from _base import HandshakeError
from _base import build_location
from _base import validate_protocol


_MANDATORY_HEADERS = [
    # key, expected value or None
    ['Upgrade', 'WebSocket'],
    ['Connection', 'Upgrade'],
    ['Host', None],
    ['Origin', None],
]

_FIRST_FIVE_LINES = map(re.compile, [
    r'^GET /[\S]* HTTP/1.1\r\n$',
    r'^Upgrade: WebSocket\r\n$',
    r'^Connection: Upgrade\r\n$',
    r'^Host: [\S]+\r\n$',
    r'^Origin: [\S]+\r\n$',
])

_SIXTH_AND_LATER = re.compile(
    r'^'
    r'(WebSocket-Protocol: [\x20-\x7e]+\r\n)?'
    r'(Cookie: [^\r]*\r\n)*'
    r'(Cookie2: [^\r]*\r\n)?'
    r'(Cookie: [^\r]*\r\n)*'
    r'\r\n')


class Handshaker(object):
    """This class performs Web Socket handshake."""

    def __init__(self, request, strict=False):
        """Construct an instance.

        Args:
            request: mod_python request.
            strict: Strictly check handshake request. Default: False.
                If True, request.connection must provide get_memorized_lines
                method.

        Handshaker will add attributes such as ws_resource in performing
        handshake.
        """

        self._request = request
        self._strict = strict

    def do_handshake(self):
        """Perform Web Socket Handshake."""

        self._check_header_lines()
        self._set_resource()
        self._set_origin()
        self._set_location()
        self._set_protocol()
        self._send_handshake()

    def _set_resource(self):
        self._request.ws_resource = self._request.uri

    def _set_origin(self):
        self._request.ws_origin = self._request.headers_in['Origin']

    def _set_location(self):
        self._request.ws_location = build_location(self._request)

    def _set_protocol(self):
        protocol = self._request.headers_in.get('WebSocket-Protocol')
        if protocol is not None:
            validate_protocol(protocol)
        self._request.ws_protocol = protocol

    def _send_handshake(self):
        self._request.connection.write(
                'HTTP/1.1 101 Web Socket Protocol Handshake\r\n')
        self._request.connection.write('Upgrade: WebSocket\r\n')
        self._request.connection.write('Connection: Upgrade\r\n')
        self._request.connection.write('WebSocket-Origin: ')
        self._request.connection.write(self._request.ws_origin)
        self._request.connection.write('\r\n')
        self._request.connection.write('WebSocket-Location: ')
        self._request.connection.write(self._request.ws_location)
        self._request.connection.write('\r\n')
        if self._request.ws_protocol:
            self._request.connection.write('WebSocket-Protocol: ')
            self._request.connection.write(self._request.ws_protocol)
            self._request.connection.write('\r\n')
        self._request.connection.write('\r\n')

    def _check_header_lines(self):
        for key, expected_value in _MANDATORY_HEADERS:
            actual_value = self._request.headers_in.get(key)
            if not actual_value:
                raise HandshakeError('Header %s is not defined' % key)
            if expected_value:
                if actual_value != expected_value:
                    raise HandshakeError('Illegal value for header %s: %s' %
                                         (key, actual_value))
        if self._strict:
            try:
                lines = self._request.connection.get_memorized_lines()
            except AttributeError, e:
                raise AttributeError(
                    'Strict handshake is specified but the connection '
                    'doesn\'t provide get_memorized_lines()')
            self._check_first_lines(lines)

    def _check_first_lines(self, lines):
        if len(lines) < len(_FIRST_FIVE_LINES):
            raise HandshakeError('Too few header lines: %d' % len(lines))
        for line, regexp in zip(lines, _FIRST_FIVE_LINES):
            if not regexp.search(line):
                raise HandshakeError('Unexpected header: %r doesn\'t match %r'
                                     % (line, regexp.pattern))
        sixth_and_later = ''.join(lines[5:])
        if not _SIXTH_AND_LATER.search(sixth_and_later):
            raise HandshakeError('Unexpected header: %r doesn\'t match %r'
                                 % (sixth_and_later,
                                    _SIXTH_AND_LATER.pattern))


# vi:sts=4 sw=4 et
